"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRootLogger = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const bunyan_1 = __importDefault(require("bunyan"));
const ts = __importStar(require("@terascope/utils"));
function getLogLevel(level) {
    // Set the same level for all logging types.
    if (typeof level === 'string') {
        return {
            console: level,
            file: level
        };
    }
    // Otherwise there may be a list of separate settins for each type.
    return level.reduce((prev, curr) => {
        Object.assign(prev, curr);
        return prev;
    }, {});
}
function createRootLogger(context) {
    const useDebugLogger = ts.toBoolean(process.env.USE_DEBUG_LOGGER || ts.isTest);
    const filename = context.name;
    const name = context.assignment || filename;
    const foundationConfig = context.sysconfig.terafoundation;
    const logLevel = getLogLevel(foundationConfig.log_level);
    if (useDebugLogger) {
        return ts.debugLogger(`${filename}:${name}`);
    }
    const streamConfig = [];
    const { environment = 'development' } = foundationConfig;
    // Setup console logging. Always turned on for development but off by
    // default for production.
    if (environment === 'development' || ts.includes(foundationConfig.logging, 'console')) {
        const level = logLevel.console ? logLevel.console : 'info';
        streamConfig.push({ stream: process.stdout, level });
    }
    // Setup logging to files.
    if (ts.includes(foundationConfig.logging, 'file')) {
        const configPath = foundationConfig.log_path || './logs';
        // remove whitespace
        const logfile = filename.trim();
        try {
            // Verify the path is a directory by resolving any symlinks
            const pathCheck = fs_1.default.lstatSync(fs_1.default.realpathSync(configPath));
            if (pathCheck.isDirectory()) {
                const level = logLevel.file ? logLevel.file : 'info';
                streamConfig.push({ path: path_1.default.join(configPath, `${logfile}.log`), level });
            }
            else {
                // This is error is just caught by the catch block below.
                throw new Error(`${configPath} is not a directory`);
            }
        }
        catch (e) {
            throw new Error(`Could not write to log_path: ${configPath}`);
        }
    }
    const loggerConfig = {
        name: filename,
        streams: streamConfig,
        assignment: context.assignment,
    };
    const logger = bunyan_1.default.createLogger(loggerConfig);
    logger.flush = async () => true;
    return logger;
}
exports.createRootLogger = createRootLogger;
//# sourceMappingURL=utils.js.map