"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClusterContext = void 0;
const cluster_1 = __importDefault(require("cluster"));
const ts = __importStar(require("@terascope/utils"));
const sysconfig_1 = require("./sysconfig");
const validate_configs_1 = __importDefault(require("./validate-configs"));
const core_context_1 = require("./core-context");
const master_1 = __importDefault(require("./master"));
const cluster = cluster_1.default;
// this module is not really testable
/* istanbul ignore next */
/**
 * A Cluster Context (with worker processes), useful for scaling
*/
class ClusterContext extends core_context_1.CoreContext {
    constructor(config) {
        const parsedArgs = sysconfig_1.getArgs(config.default_config_file);
        const sysconfig = validate_configs_1.default(cluster, config, parsedArgs.configfile);
        super(config, cluster, sysconfig);
        this._errorHandler = this._errorHandler.bind(this);
        core_context_1.handleStdStreams();
        process.on('uncaughtException', this._errorHandler);
        process.on('unhandledRejection', this._errorHandler);
        if (config.script) {
            /**
             * Run a script only
             */
            config.script(this);
        }
        else if (this.cluster.isMaster) {
            /**
             * Use cluster to start multiple workers
             */
            master_1.default(this, config);
            if (ts.isFunction(config.master)) {
                config.master(this, config);
            }
        }
        else {
            /**
             * Start a worker process
             */
            let keyFound = false;
            if (config.descriptors) {
                Object.keys(config.descriptors).forEach((key) => {
                    if (this.assignment === key) {
                        keyFound = true;
                        config[key](this);
                    }
                });
                // if no key was explicitly set then default to worker
                if (!keyFound) {
                    config.worker(this);
                }
            }
            else {
                config.worker(this);
            }
        }
    }
    _errorHandler(err) {
        // eslint-disable-next-line no-console
        const logErr = this.logger
            ? this.logger.error.bind(this.logger)
            : console.error;
        if (cluster.isMaster) {
            logErr(ts.getFullErrorStack(err), `Error in master with pid: ${process.pid}`);
        }
        else {
            logErr(ts.getFullErrorStack(err), `Error in worker: ${ts.get(this.cluster, 'worker.id')} pid: ${process.pid}`);
        }
        // log saving to disk is async, using hack to give time to flush
        setTimeout(() => {
            process.exit(-1);
        }, 600);
    }
}
exports.ClusterContext = ClusterContext;
//# sourceMappingURL=cluster-context.js.map